@@ -1,3 +1,5 @@
+'use strict';
+
 var tap = require('tap');
 var test = tap.test;
 var semver = require('../semver.js');
@@ -11,6 +13,7 @@ var lte = semver.lte;
 var satisfies = semver.satisfies;
 var validRange = semver.validRange;
 var inc = semver.inc;
+var diff = semver.diff;
 var replaceStars = semver.replaceStars;
 var toComparators = semver.toComparators;
 var SemVer = semver.SemVer;
@@ -47,7 +50,9 @@ test('\ncomparison tests', function(t) {
     ['1.2.3-a.10', '1.2.3-a.5'],
     ['1.2.3-a.b', '1.2.3-a.5'],
     ['1.2.3-a.b', '1.2.3-a'],
-    ['1.2.3-a.b.c.10.d.5', '1.2.3-a.b.c.5.d.100']
+    ['1.2.3-a.b.c.10.d.5', '1.2.3-a.b.c.5.d.100'],
+    ['1.2.3-r2', '1.2.3-r100'],
+    ['1.2.3-r100', '1.2.3-R2']
   ].forEach(function(v) {
     var v0 = v[0];
     var v1 = v[1];
@@ -130,11 +135,20 @@ test('\nrange tests', function(t) {
   // [range, version]
   // version should be included by range
   [['1.0.0 - 2.0.0', '1.2.3'],
+    ['^1.2.3+build', '1.2.3'],
+    ['^1.2.3+build', '1.3.0'],
+    ['1.2.3-pre+asdf - 2.4.3-pre+asdf', '1.2.3'],
+    ['1.2.3pre+asdf - 2.4.3-pre+asdf', '1.2.3', true],
+    ['1.2.3-pre+asdf - 2.4.3pre+asdf', '1.2.3', true],
+    ['1.2.3pre+asdf - 2.4.3pre+asdf', '1.2.3', true],
+    ['1.2.3-pre+asdf - 2.4.3-pre+asdf', '1.2.3-pre.2'],
+    ['1.2.3-pre+asdf - 2.4.3-pre+asdf', '2.4.3-alpha'],
+    ['1.2.3+asdf - 2.4.3+asdf', '1.2.3'],
     ['1.0.0', '1.0.0'],
     ['>=*', '0.2.4'],
     ['', '1.0.0'],
     ['*', '1.2.3'],
-    ['*', 'v1.2.3-foo', true],
+    ['*', 'v1.2.3', true],
     ['>=1.0.0', '1.0.0'],
     ['>=1.0.0', '1.0.1'],
     ['>=1.0.0', '1.1.0'],
@@ -187,13 +201,11 @@ test('\nrange tests', function(t) {
     ['>= 1', '1.0.0'],
     ['<1.2', '1.1.1'],
     ['< 1.2', '1.1.1'],
-    ['1', '1.0.0beta', true],
     ['~v0.5.4-pre', '0.5.5'],
     ['~v0.5.4-pre', '0.5.4'],
     ['=0.7.x', '0.7.2'],
+    ['<=0.7.x', '0.7.2'],
     ['>=0.7.x', '0.7.2'],
-    ['=0.7.x', '0.7.0-asdf'],
-    ['>=0.7.x', '0.7.0-asdf'],
     ['<=0.7.x', '0.6.2'],
     ['~1.2.1 >=1.2.3', '1.2.3'],
     ['~1.2.1 =1.2.3', '1.2.3'],
@@ -205,17 +217,15 @@ test('\nrange tests', function(t) {
     ['1.2.3 >=1.2.1', '1.2.3'],
     ['>=1.2.3 >=1.2.1', '1.2.3'],
     ['>=1.2.1 >=1.2.3', '1.2.3'],
-    ['<=1.2.3', '1.2.3-beta'],
-    ['>1.2', '1.3.0-beta'],
     ['>=1.2', '1.2.8'],
     ['^1.2.3', '1.8.1'],
-    ['^1.2.3', '1.2.3-beta'],
     ['^0.1.2', '0.1.2'],
     ['^0.1', '0.1.2'],
     ['^1.2', '1.4.2'],
     ['^1.2 ^1', '1.4.2'],
-    ['^1.2', '1.2.0-pre'],
-    ['^1.2.3', '1.2.3-pre']
+    ['^1.2.3-alpha', '1.2.3-pre'],
+    ['^1.2.0-alpha', '1.2.0-pre'],
+    ['^0.0.1-alpha', '0.0.1-beta']
   ].forEach(function(v) {
     var range = v[0];
     var ver = v[1];
@@ -229,6 +239,20 @@ test('\nnegative range tests', function(t) {
   // [range, version]
   // version should not be included by range
   [['1.0.0 - 2.0.0', '2.2.3'],
+    ['1.2.3+asdf - 2.4.3+asdf', '1.2.3-pre.2'],
+    ['1.2.3+asdf - 2.4.3+asdf', '2.4.3-alpha'],
+    ['^1.2.3+build', '2.0.0'],
+    ['^1.2.3+build', '1.2.0'],
+    ['^1.2.3', '1.2.3-pre'],
+    ['^1.2', '1.2.0-pre'],
+    ['>1.2', '1.3.0-beta'],
+    ['<=1.2.3', '1.2.3-beta'],
+    ['^1.2.3', '1.2.3-beta'],
+    ['=0.7.x', '0.7.0-asdf'],
+    ['>=0.7.x', '0.7.0-asdf'],
+    ['1', '1.0.0beta', true],
+    ['<1', '1.0.0beta', true],
+    ['< 1', '1.0.0beta', true],
     ['1.0.0', '1.0.1'],
     ['>=1.0.0', '0.0.0'],
     ['>=1.0.0', '0.0.1'],
@@ -268,17 +292,16 @@ test('\nnegative range tests', function(t) {
     ['>=1.2', '1.1.1'],
     ['1', '2.0.0beta', true],
     ['~v0.5.4-beta', '0.5.4-alpha'],
-    ['<1', '1.0.0beta', true],
-    ['< 1', '1.0.0beta', true],
     ['=0.7.x', '0.8.2'],
     ['>=0.7.x', '0.6.2'],
-    ['<=0.7.x', '0.7.2'],
+    ['<0.7.x', '0.7.2'],
     ['<1.2.3', '1.2.3-beta'],
     ['=1.2.3', '1.2.3-beta'],
     ['>1.2', '1.2.8'],
     ['^1.2.3', '2.0.0-alpha'],
     ['^1.2.3', '1.2.2'],
     ['^1.2', '1.1.9'],
+    ['*', 'v1.2.3-foo', true],
     // invalid ranges never satisfied!
     ['blerg', '1.2.3'],
     ['git+https://user:password0123@github.com/foo', '123.0.0', true],
@@ -294,16 +317,23 @@ test('\nnegative range tests', function(t) {
 });
 
 test('\nincrement versions test', function(t) {
-  // [version, inc, result]
-  // inc(version, inc) -> result
+//  [version, inc, result, identifier]
+//  inc(version, inc) -> result
   [['1.2.3', 'major', '2.0.0'],
     ['1.2.3', 'minor', '1.3.0'],
     ['1.2.3', 'patch', '1.2.4'],
     ['1.2.3tag', 'major', '2.0.0', true],
     ['1.2.3-tag', 'major', '2.0.0'],
     ['1.2.3', 'fake', null],
+    ['1.2.0-0', 'patch', '1.2.0'],
     ['fake', 'major', null],
-    ['1.2.3', 'prerelease', '1.2.3-0'],
+    ['1.2.3-4', 'major', '2.0.0'],
+    ['1.2.3-4', 'minor', '1.3.0'],
+    ['1.2.3-4', 'patch', '1.2.3'],
+    ['1.2.3-alpha.0.beta', 'major', '2.0.0'],
+    ['1.2.3-alpha.0.beta', 'minor', '1.3.0'],
+    ['1.2.3-alpha.0.beta', 'patch', '1.2.3'],
+    ['1.2.4', 'prerelease', '1.2.5-0'],
     ['1.2.3-0', 'prerelease', '1.2.3-1'],
     ['1.2.3-alpha.0', 'prerelease', '1.2.3-alpha.1'],
     ['1.2.3-alpha.1', 'prerelease', '1.2.3-alpha.2'],
@@ -319,14 +349,108 @@ test('\nincrement versions test', function(t) {
     ['1.2.3-alpha.10.beta.2', 'prerelease', '1.2.3-alpha.10.beta.3'],
     ['1.2.3-alpha.9.beta', 'prerelease', '1.2.3-alpha.10.beta'],
     ['1.2.3-alpha.10.beta', 'prerelease', '1.2.3-alpha.11.beta'],
-    ['1.2.3-alpha.11.beta', 'prerelease', '1.2.3-alpha.12.beta']
+    ['1.2.3-alpha.11.beta', 'prerelease', '1.2.3-alpha.12.beta'],
+    ['1.2.0', 'prepatch', '1.2.1-0'],
+    ['1.2.0-1', 'prepatch', '1.2.1-0'],
+    ['1.2.0', 'preminor', '1.3.0-0'],
+    ['1.2.3-1', 'preminor', '1.3.0-0'],
+    ['1.2.0', 'premajor', '2.0.0-0'],
+    ['1.2.3-1', 'premajor', '2.0.0-0'],
+    ['1.2.0-1', 'minor', '1.2.0'],
+    ['1.0.0-1', 'major', '1.0.0'],
+
+    ['1.2.3', 'major', '2.0.0', false, 'dev'],
+    ['1.2.3', 'minor', '1.3.0', false, 'dev'],
+    ['1.2.3', 'patch', '1.2.4', false, 'dev'],
+    ['1.2.3tag', 'major', '2.0.0', true, 'dev'],
+    ['1.2.3-tag', 'major', '2.0.0', false, 'dev'],
+    ['1.2.3', 'fake', null, false, 'dev'],
+    ['1.2.0-0', 'patch', '1.2.0', false, 'dev'],
+    ['fake', 'major', null, false, 'dev'],
+    ['1.2.3-4', 'major', '2.0.0', false, 'dev'],
+    ['1.2.3-4', 'minor', '1.3.0', false, 'dev'],
+    ['1.2.3-4', 'patch', '1.2.3', false, 'dev'],
+    ['1.2.3-alpha.0.beta', 'major', '2.0.0', false, 'dev'],
+    ['1.2.3-alpha.0.beta', 'minor', '1.3.0', false, 'dev'],
+    ['1.2.3-alpha.0.beta', 'patch', '1.2.3', false, 'dev'],
+    ['1.2.4', 'prerelease', '1.2.5-dev.0', false, 'dev'],
+    ['1.2.3-0', 'prerelease', '1.2.3-dev.0', false, 'dev'],
+    ['1.2.3-alpha.0', 'prerelease', '1.2.3-dev.0', false, 'dev'],
+    ['1.2.3-alpha.0', 'prerelease', '1.2.3-alpha.1', false, 'alpha'],
+    ['1.2.3-alpha.0.beta', 'prerelease', '1.2.3-dev.0', false, 'dev'],
+    ['1.2.3-alpha.0.beta', 'prerelease', '1.2.3-alpha.1.beta', false, 'alpha'],
+    ['1.2.3-alpha.10.0.beta', 'prerelease', '1.2.3-dev.0', false, 'dev'],
+    ['1.2.3-alpha.10.0.beta', 'prerelease', '1.2.3-alpha.10.1.beta', false, 'alpha'],
+    ['1.2.3-alpha.10.1.beta', 'prerelease', '1.2.3-alpha.10.2.beta', false, 'alpha'],
+    ['1.2.3-alpha.10.2.beta', 'prerelease', '1.2.3-alpha.10.3.beta', false, 'alpha'],
+    ['1.2.3-alpha.10.beta.0', 'prerelease', '1.2.3-dev.0', false, 'dev'],
+    ['1.2.3-alpha.10.beta.0', 'prerelease', '1.2.3-alpha.10.beta.1', false, 'alpha'],
+    ['1.2.3-alpha.10.beta.1', 'prerelease', '1.2.3-alpha.10.beta.2', false, 'alpha'],
+    ['1.2.3-alpha.10.beta.2', 'prerelease', '1.2.3-alpha.10.beta.3', false, 'alpha'],
+    ['1.2.3-alpha.9.beta', 'prerelease', '1.2.3-dev.0', false, 'dev'],
+    ['1.2.3-alpha.9.beta', 'prerelease', '1.2.3-alpha.10.beta', false, 'alpha'],
+    ['1.2.3-alpha.10.beta', 'prerelease', '1.2.3-alpha.11.beta', false, 'alpha'],
+    ['1.2.3-alpha.11.beta', 'prerelease', '1.2.3-alpha.12.beta', false, 'alpha'],
+    ['1.2.0', 'prepatch', '1.2.1-dev.0', false, 'dev'],
+    ['1.2.0-1', 'prepatch', '1.2.1-dev.0', false, 'dev'],
+    ['1.2.0', 'preminor', '1.3.0-dev.0', false, 'dev'],
+    ['1.2.3-1', 'preminor', '1.3.0-dev.0', false, 'dev'],
+    ['1.2.0', 'premajor', '2.0.0-dev.0', false, 'dev'],
+    ['1.2.3-1', 'premajor', '2.0.0-dev.0', false, 'dev'],
+    ['1.2.0-1', 'minor', '1.2.0', false, 'dev'],
+    ['1.0.0-1', 'major', '1.0.0', false, 'dev'],
+    ['1.2.3-dev.bar', 'prerelease', '1.2.3-dev.0', false, 'dev']
+
   ].forEach(function(v) {
     var pre = v[0];
     var what = v[1];
     var wanted = v[2];
     var loose = v[3];
-    var found = inc(pre, what, loose);
-    t.equal(found, wanted, 'inc(' + pre + ', ' + what + ') === ' + wanted);
+    var id = v[4];
+    var found = inc(pre, what, loose, id);
+    var cmd = 'inc(' + pre + ', ' + what + ', ' + id + ')';
+    t.equal(found, wanted, cmd + ' === ' + wanted);
+
+    var parsed = semver.parse(pre, loose);
+    if (wanted) {
+      parsed.inc(what, id);
+      t.equal(parsed.version, wanted, cmd + ' object version updated');
+      t.equal(parsed.raw, wanted, cmd + ' object raw field updated');
+    } else if (parsed) {
+      t.throws(function () {
+        parsed.inc(what, id)
+      })
+    } else {
+      t.equal(parsed, null)
+    }
+  });
+
+  t.end();
+});
+
+test('\ndiff versions test', function(t) {
+//  [version1, version2, result]
+//  diff(version1, version2) -> result
+  [['1.2.3', '0.2.3', 'major'],
+    ['1.4.5', '0.2.3', 'major'],
+    ['1.2.3', '2.0.0-pre', 'premajor'],
+    ['1.2.3', '1.3.3', 'minor'],
+    ['1.0.1', '1.1.0-pre', 'preminor'],
+    ['1.2.3', '1.2.4', 'patch'],
+    ['1.2.3', '1.2.4-pre', 'prepatch'],
+    ['0.0.1', '0.0.1-pre', 'prerelease'],
+    ['0.0.1', '0.0.1-pre-2', 'prerelease'],
+    ['1.1.0', '1.1.0-pre', 'prerelease'],
+    ['1.1.0-pre-1', '1.1.0-pre-2', 'prerelease'],
+    ['1.0.0', '1.0.0', null]
+
+  ].forEach(function(v) {
+    var version1 = v[0];
+    var version2 = v[1];
+    var wanted = v[2];
+    var found = diff(version1, version2);
+    var cmd = 'diff(' + version1 + ', ' + version2 + ')';
+    t.equal(found, wanted, cmd + ' === ' + wanted);
   });
 
   t.end();
@@ -338,18 +462,18 @@ test('\nvalid range test', function(t) {
   // translate ranges into their canonical form
   [['1.0.0 - 2.0.0', '>=1.0.0 <=2.0.0'],
     ['1.0.0', '1.0.0'],
-    ['>=*', '>=0.0.0-0'],
+    ['>=*', '*'],
     ['', '*'],
     ['*', '*'],
     ['*', '*'],
     ['>=1.0.0', '>=1.0.0'],
     ['>1.0.0', '>1.0.0'],
     ['<=2.0.0', '<=2.0.0'],
-    ['1', '>=1.0.0-0 <2.0.0-0'],
+    ['1', '>=1.0.0 <2.0.0'],
     ['<=2.0.0', '<=2.0.0'],
     ['<=2.0.0', '<=2.0.0'],
-    ['<2.0.0', '<2.0.0-0'],
-    ['<2.0.0', '<2.0.0-0'],
+    ['<2.0.0', '<2.0.0'],
+    ['<2.0.0', '<2.0.0'],
     ['>= 1.0.0', '>=1.0.0'],
     ['>=  1.0.0', '>=1.0.0'],
     ['>=   1.0.0', '>=1.0.0'],
@@ -358,56 +482,56 @@ test('\nvalid range test', function(t) {
     ['<=   2.0.0', '<=2.0.0'],
     ['<= 2.0.0', '<=2.0.0'],
     ['<=  2.0.0', '<=2.0.0'],
-    ['<    2.0.0', '<2.0.0-0'],
-    ['<	2.0.0', '<2.0.0-0'],
+    ['<    2.0.0', '<2.0.0'],
+    ['<	2.0.0', '<2.0.0'],
     ['>=0.1.97', '>=0.1.97'],
     ['>=0.1.97', '>=0.1.97'],
     ['0.1.20 || 1.2.4', '0.1.20||1.2.4'],
-    ['>=0.2.3 || <0.0.1', '>=0.2.3||<0.0.1-0'],
-    ['>=0.2.3 || <0.0.1', '>=0.2.3||<0.0.1-0'],
-    ['>=0.2.3 || <0.0.1', '>=0.2.3||<0.0.1-0'],
+    ['>=0.2.3 || <0.0.1', '>=0.2.3||<0.0.1'],
+    ['>=0.2.3 || <0.0.1', '>=0.2.3||<0.0.1'],
+    ['>=0.2.3 || <0.0.1', '>=0.2.3||<0.0.1'],
     ['||', '||'],
-    ['2.x.x', '>=2.0.0-0 <3.0.0-0'],
-    ['1.2.x', '>=1.2.0-0 <1.3.0-0'],
-    ['1.2.x || 2.x', '>=1.2.0-0 <1.3.0-0||>=2.0.0-0 <3.0.0-0'],
-    ['1.2.x || 2.x', '>=1.2.0-0 <1.3.0-0||>=2.0.0-0 <3.0.0-0'],
+    ['2.x.x', '>=2.0.0 <3.0.0'],
+    ['1.2.x', '>=1.2.0 <1.3.0'],
+    ['1.2.x || 2.x', '>=1.2.0 <1.3.0||>=2.0.0 <3.0.0'],
+    ['1.2.x || 2.x', '>=1.2.0 <1.3.0||>=2.0.0 <3.0.0'],
     ['x', '*'],
-    ['2.*.*', '>=2.0.0-0 <3.0.0-0'],
-    ['1.2.*', '>=1.2.0-0 <1.3.0-0'],
-    ['1.2.* || 2.*', '>=1.2.0-0 <1.3.0-0||>=2.0.0-0 <3.0.0-0'],
+    ['2.*.*', '>=2.0.0 <3.0.0'],
+    ['1.2.*', '>=1.2.0 <1.3.0'],
+    ['1.2.* || 2.*', '>=1.2.0 <1.3.0||>=2.0.0 <3.0.0'],
     ['*', '*'],
-    ['2', '>=2.0.0-0 <3.0.0-0'],
-    ['2.3', '>=2.3.0-0 <2.4.0-0'],
-    ['~2.4', '>=2.4.0-0 <2.5.0-0'],
-    ['~2.4', '>=2.4.0-0 <2.5.0-0'],
-    ['~>3.2.1', '>=3.2.1-0 <3.3.0-0'],
-    ['~1', '>=1.0.0-0 <2.0.0-0'],
-    ['~>1', '>=1.0.0-0 <2.0.0-0'],
-    ['~> 1', '>=1.0.0-0 <2.0.0-0'],
-    ['~1.0', '>=1.0.0-0 <1.1.0-0'],
-    ['~ 1.0', '>=1.0.0-0 <1.1.0-0'],
-    ['^0', '>=0.0.0-0 <1.0.0-0'],
-    ['^ 1', '>=1.0.0-0 <2.0.0-0'],
-    ['^0.1', '>=0.1.0-0 <0.2.0-0'],
-    ['^1.0', '>=1.0.0-0 <2.0.0-0'],
-    ['^1.2', '>=1.2.0-0 <2.0.0-0'],
-    ['^0.0.1', '=0.0.1'],
-    ['^0.0.1-beta', '=0.0.1-beta'],
-    ['^0.1.2', '>=0.1.2-0 <0.2.0-0'],
-    ['^1.2.3', '>=1.2.3-0 <2.0.0-0'],
-    ['^1.2.3-beta.4', '>=1.2.3-beta.4 <2.0.0-0'],
-    ['<1', '<1.0.0-0'],
-    ['< 1', '<1.0.0-0'],
-    ['>=1', '>=1.0.0-0'],
-    ['>= 1', '>=1.0.0-0'],
-    ['<1.2', '<1.2.0-0'],
-    ['< 1.2', '<1.2.0-0'],
-    ['1', '>=1.0.0-0 <2.0.0-0'],
+    ['2', '>=2.0.0 <3.0.0'],
+    ['2.3', '>=2.3.0 <2.4.0'],
+    ['~2.4', '>=2.4.0 <2.5.0'],
+    ['~2.4', '>=2.4.0 <2.5.0'],
+    ['~>3.2.1', '>=3.2.1 <3.3.0'],
+    ['~1', '>=1.0.0 <2.0.0'],
+    ['~>1', '>=1.0.0 <2.0.0'],
+    ['~> 1', '>=1.0.0 <2.0.0'],
+    ['~1.0', '>=1.0.0 <1.1.0'],
+    ['~ 1.0', '>=1.0.0 <1.1.0'],
+    ['^0', '>=0.0.0 <1.0.0'],
+    ['^ 1', '>=1.0.0 <2.0.0'],
+    ['^0.1', '>=0.1.0 <0.2.0'],
+    ['^1.0', '>=1.0.0 <2.0.0'],
+    ['^1.2', '>=1.2.0 <2.0.0'],
+    ['^0.0.1', '>=0.0.1 <0.0.2'],
+    ['^0.0.1-beta', '>=0.0.1-beta <0.0.2'],
+    ['^0.1.2', '>=0.1.2 <0.2.0'],
+    ['^1.2.3', '>=1.2.3 <2.0.0'],
+    ['^1.2.3-beta.4', '>=1.2.3-beta.4 <2.0.0'],
+    ['<1', '<1.0.0'],
+    ['< 1', '<1.0.0'],
+    ['>=1', '>=1.0.0'],
+    ['>= 1', '>=1.0.0'],
+    ['<1.2', '<1.2.0'],
+    ['< 1.2', '<1.2.0'],
+    ['1', '>=1.0.0 <2.0.0'],
     ['>01.02.03', '>1.2.3', true],
     ['>01.02.03', null],
-    ['~1.2.3beta', '>=1.2.3-beta <1.3.0-0', true],
+    ['~1.2.3beta', '>=1.2.3-beta <1.3.0', true],
     ['~1.2.3beta', null],
-    ['^ 1.2 ^ 1', '>=1.2.0-0 <2.0.0-0 >=1.0.0-0 <2.0.0-0']
+    ['^ 1.2 ^ 1', '>=1.2.0 <2.0.0 >=1.0.0 <2.0.0']
   ].forEach(function(v) {
     var pre = v[0];
     var wanted = v[1];
@@ -425,7 +549,7 @@ test('\ncomparators test', function(t) {
   // turn range into a set of individual comparators
   [['1.0.0 - 2.0.0', [['>=1.0.0', '<=2.0.0']]],
     ['1.0.0', [['1.0.0']]],
-    ['>=*', [['>=0.0.0-0']]],
+    ['>=*', [['']]],
     ['', [['']]],
     ['*', [['']]],
     ['*', [['']]],
@@ -435,11 +559,11 @@ test('\ncomparators test', function(t) {
     ['>1.0.0', [['>1.0.0']]],
     ['>1.0.0', [['>1.0.0']]],
     ['<=2.0.0', [['<=2.0.0']]],
-    ['1', [['>=1.0.0-0', '<2.0.0-0']]],
+    ['1', [['>=1.0.0', '<2.0.0']]],
     ['<=2.0.0', [['<=2.0.0']]],
     ['<=2.0.0', [['<=2.0.0']]],
-    ['<2.0.0', [['<2.0.0-0']]],
-    ['<2.0.0', [['<2.0.0-0']]],
+    ['<2.0.0', [['<2.0.0']]],
+    ['<2.0.0', [['<2.0.0']]],
     ['>= 1.0.0', [['>=1.0.0']]],
     ['>=  1.0.0', [['>=1.0.0']]],
     ['>=   1.0.0', [['>=1.0.0']]],
@@ -448,47 +572,50 @@ test('\ncomparators test', function(t) {
     ['<=   2.0.0', [['<=2.0.0']]],
     ['<= 2.0.0', [['<=2.0.0']]],
     ['<=  2.0.0', [['<=2.0.0']]],
-    ['<    2.0.0', [['<2.0.0-0']]],
-    ['<\t2.0.0', [['<2.0.0-0']]],
+    ['<    2.0.0', [['<2.0.0']]],
+    ['<\t2.0.0', [['<2.0.0']]],
     ['>=0.1.97', [['>=0.1.97']]],
     ['>=0.1.97', [['>=0.1.97']]],
     ['0.1.20 || 1.2.4', [['0.1.20'], ['1.2.4']]],
-    ['>=0.2.3 || <0.0.1', [['>=0.2.3'], ['<0.0.1-0']]],
-    ['>=0.2.3 || <0.0.1', [['>=0.2.3'], ['<0.0.1-0']]],
-    ['>=0.2.3 || <0.0.1', [['>=0.2.3'], ['<0.0.1-0']]],
+    ['>=0.2.3 || <0.0.1', [['>=0.2.3'], ['<0.0.1']]],
+    ['>=0.2.3 || <0.0.1', [['>=0.2.3'], ['<0.0.1']]],
+    ['>=0.2.3 || <0.0.1', [['>=0.2.3'], ['<0.0.1']]],
     ['||', [[''], ['']]],
-    ['2.x.x', [['>=2.0.0-0', '<3.0.0-0']]],
-    ['1.2.x', [['>=1.2.0-0', '<1.3.0-0']]],
-    ['1.2.x || 2.x', [['>=1.2.0-0', '<1.3.0-0'], ['>=2.0.0-0', '<3.0.0-0']]],
-    ['1.2.x || 2.x', [['>=1.2.0-0', '<1.3.0-0'], ['>=2.0.0-0', '<3.0.0-0']]],
+    ['2.x.x', [['>=2.0.0', '<3.0.0']]],
+    ['1.2.x', [['>=1.2.0', '<1.3.0']]],
+    ['1.2.x || 2.x', [['>=1.2.0', '<1.3.0'], ['>=2.0.0', '<3.0.0']]],
+    ['1.2.x || 2.x', [['>=1.2.0', '<1.3.0'], ['>=2.0.0', '<3.0.0']]],
     ['x', [['']]],
-    ['2.*.*', [['>=2.0.0-0', '<3.0.0-0']]],
-    ['1.2.*', [['>=1.2.0-0', '<1.3.0-0']]],
-    ['1.2.* || 2.*', [['>=1.2.0-0', '<1.3.0-0'], ['>=2.0.0-0', '<3.0.0-0']]],
-    ['1.2.* || 2.*', [['>=1.2.0-0', '<1.3.0-0'], ['>=2.0.0-0', '<3.0.0-0']]],
+    ['2.*.*', [['>=2.0.0', '<3.0.0']]],
+    ['1.2.*', [['>=1.2.0', '<1.3.0']]],
+    ['1.2.* || 2.*', [['>=1.2.0', '<1.3.0'], ['>=2.0.0', '<3.0.0']]],
+    ['1.2.* || 2.*', [['>=1.2.0', '<1.3.0'], ['>=2.0.0', '<3.0.0']]],
     ['*', [['']]],
-    ['2', [['>=2.0.0-0', '<3.0.0-0']]],
-    ['2.3', [['>=2.3.0-0', '<2.4.0-0']]],
-    ['~2.4', [['>=2.4.0-0', '<2.5.0-0']]],
-    ['~2.4', [['>=2.4.0-0', '<2.5.0-0']]],
-    ['~>3.2.1', [['>=3.2.1-0', '<3.3.0-0']]],
-    ['~1', [['>=1.0.0-0', '<2.0.0-0']]],
-    ['~>1', [['>=1.0.0-0', '<2.0.0-0']]],
-    ['~> 1', [['>=1.0.0-0', '<2.0.0-0']]],
-    ['~1.0', [['>=1.0.0-0', '<1.1.0-0']]],
-    ['~ 1.0', [['>=1.0.0-0', '<1.1.0-0']]],
-    ['~ 1.0.3', [['>=1.0.3-0', '<1.1.0-0']]],
-    ['~> 1.0.3', [['>=1.0.3-0', '<1.1.0-0']]],
-    ['<1', [['<1.0.0-0']]],
-    ['< 1', [['<1.0.0-0']]],
-    ['>=1', [['>=1.0.0-0']]],
-    ['>= 1', [['>=1.0.0-0']]],
-    ['<1.2', [['<1.2.0-0']]],
-    ['< 1.2', [['<1.2.0-0']]],
-    ['1', [['>=1.0.0-0', '<2.0.0-0']]],
-    ['1 2', [['>=1.0.0-0', '<2.0.0-0', '>=2.0.0-0', '<3.0.0-0']]],
-    ['1.2 - 3.4.5', [['>=1.2.0-0', '<=3.4.5']]],
-    ['1.2.3 - 3.4', [['>=1.2.3', '<3.5.0-0']]]
+    ['2', [['>=2.0.0', '<3.0.0']]],
+    ['2.3', [['>=2.3.0', '<2.4.0']]],
+    ['~2.4', [['>=2.4.0', '<2.5.0']]],
+    ['~2.4', [['>=2.4.0', '<2.5.0']]],
+    ['~>3.2.1', [['>=3.2.1', '<3.3.0']]],
+    ['~1', [['>=1.0.0', '<2.0.0']]],
+    ['~>1', [['>=1.0.0', '<2.0.0']]],
+    ['~> 1', [['>=1.0.0', '<2.0.0']]],
+    ['~1.0', [['>=1.0.0', '<1.1.0']]],
+    ['~ 1.0', [['>=1.0.0', '<1.1.0']]],
+    ['~ 1.0.3', [['>=1.0.3', '<1.1.0']]],
+    ['~> 1.0.3', [['>=1.0.3', '<1.1.0']]],
+    ['<1', [['<1.0.0']]],
+    ['< 1', [['<1.0.0']]],
+    ['>=1', [['>=1.0.0']]],
+    ['>= 1', [['>=1.0.0']]],
+    ['<1.2', [['<1.2.0']]],
+    ['< 1.2', [['<1.2.0']]],
+    ['1', [['>=1.0.0', '<2.0.0']]],
+    ['1 2', [['>=1.0.0', '<2.0.0', '>=2.0.0', '<3.0.0']]],
+    ['1.2 - 3.4.5', [['>=1.2.0', '<=3.4.5']]],
+    ['1.2.3 - 3.4', [['>=1.2.3', '<3.5.0']]],
+    ['1.2.3 - 3', [['>=1.2.3', '<4.0.0']]],
+    ['>*', [['<0.0.0']]],
+    ['<*', [['<0.0.0']]]
   ].forEach(function(v) {
     var pre = v[0];
     var wanted = v[1];
@@ -500,6 +627,21 @@ test('\ncomparators test', function(t) {
   t.end();
 });
 
+test('\ninvalid version numbers', function(t) {
+  ['1.2.3.4',
+   'NOT VALID',
+   1.2,
+   null,
+   'Infinity.NaN.Infinity'
+  ].forEach(function(v) {
+    t.throws(function() {
+      new SemVer(v);
+    }, {name:'TypeError', message:'Invalid Version: ' + v});
+  });
+
+  t.end();
+});
+
 test('\nstrict vs loose version numbers', function(t) {
   [['=1.2.3', '1.2.3'],
     ['01.02.03', '1.2.3'],
@@ -527,7 +669,7 @@ test('\nstrict vs loose version numbers', function(t) {
 
 test('\nstrict vs loose ranges', function(t) {
   [['>=01.02.03', '>=1.2.3'],
-    ['~1.02.03beta', '>=1.2.3-beta <1.3.0-0']
+    ['~1.02.03beta', '>=1.2.3-beta <1.3.0']
   ].forEach(function(v) {
     var loose = v[0];
     var comps = v[1];
@@ -542,7 +684,7 @@ test('\nstrict vs loose ranges', function(t) {
 test('\nmax satisfying', function(t) {
   [[['1.2.3', '1.2.4'], '1.2', '1.2.4'],
     [['1.2.4', '1.2.3'], '1.2', '1.2.4'],
-    [['1.2.3','1.2.4','1.2.5','1.2.6'], '~1.2.3', '1.2.6'],
+    [['1.2.3', '1.2.4', '1.2.5', '1.2.6'], '~1.2.3', '1.2.6'],
     [['1.1.0', '1.2.0', '1.2.1', '1.3.0', '2.0.0b1', '2.0.0b2', '2.0.0b3', '2.0.0', '2.1.0'], '~2.0.0', '2.0.0', true]
   ].forEach(function(v) {
     var versions = v[0];
@@ -554,3 +696,19 @@ test('\nmax satisfying', function(t) {
   });
   t.end();
 });
+
+test('\nmin satisfying', function(t) {
+  [[['1.2.3', '1.2.4'], '1.2', '1.2.3'],
+    [['1.2.4', '1.2.3'], '1.2', '1.2.3'],
+    [['1.2.3', '1.2.4', '1.2.5', '1.2.6'], '~1.2.3', '1.2.3'],
+    [['1.1.0', '1.2.0', '1.2.1', '1.3.0', '2.0.0b1', '2.0.0b2', '2.0.0b3', '2.0.0', '2.1.0'], '~2.0.0', '2.0.0', true]
+  ].forEach(function(v) {
+    var versions = v[0];
+    var range = v[1];
+    var expect = v[2];
+    var loose = v[3];
+    var actual = semver.minSatisfying(versions, range, loose);
+    t.equal(actual, expect);
+  });
+  t.end();
+});
